@@ -113,6 +113,25 @@ class Elf32File:
             common.Utils.eprint(f"Warning: Elf uses not supported architecture level: {unkArchLevelNames}")
             common.Utils.eprint(f"\t This means this elf probably uses an unknown instruction set")
 
+        if Elf32HeaderFlag.ARCH_1 in self.elfFlags:
+            common.GlobalConfig.ARCHLEVEL = common.ArchLevel.MIPS1
+        elif Elf32HeaderFlag.ARCH_2 in self.elfFlags:
+            common.GlobalConfig.ARCHLEVEL = common.ArchLevel.MIPS2
+        elif Elf32HeaderFlag.ARCH_3 in self.elfFlags:
+            common.GlobalConfig.ARCHLEVEL = common.ArchLevel.MIPS3
+        elif Elf32HeaderFlag.ARCH_4 in self.elfFlags:
+            common.GlobalConfig.ARCHLEVEL = common.ArchLevel.MIPS5
+        elif Elf32HeaderFlag.ARCH_5 in self.elfFlags:
+            common.GlobalConfig.ARCHLEVEL = common.ArchLevel.MIPS5
+        elif Elf32HeaderFlag.ARCH_32 in self.elfFlags:
+            common.GlobalConfig.ARCHLEVEL = common.ArchLevel.MIPS32
+        elif Elf32HeaderFlag.ARCH_64 in self.elfFlags:
+            common.GlobalConfig.ARCHLEVEL = common.ArchLevel.MIPS64
+        elif Elf32HeaderFlag.ARCH_32R2 in self.elfFlags:
+            common.GlobalConfig.ARCHLEVEL = common.ArchLevel.MIPS32R2
+        elif Elf32HeaderFlag.ARCH_64R2 in self.elfFlags:
+            common.GlobalConfig.ARCHLEVEL = common.ArchLevel.MIPS64R2
+
 
     def _processSection_NULL(self, array_of_bytes: bytearray, entry: Elf32SectionHeaderEntry, sectionEntryName: str) -> None:
         pass
@@ -369,12 +388,12 @@ class Elf32File:
                 if stBind is not None:
                     bind = stBind.name
 
-                visibility: str|int = sym.other
+                visibility: str = f"0x{sym.other:X}"
                 stOther = Elf32SymbolVisibility.fromValue(sym.other)
                 if stOther is not None:
                     visibility = stOther.name
 
-                ndx: str|int = sym.shndx
+                ndx: str = f"0x{sym.shndx:X}"
                 shndx = Elf32SectionHeaderNumber.fromValue(sym.shndx)
                 if shndx is not None:
                     ndx = shndx.name
@@ -422,41 +441,44 @@ class Elf32File:
             print(f"   Address {'Access':>9}  Initial Purpose")
             access = entryAddress - gpValue
             if access < 0:
-                accessStr = f"-{-access:X}"
+                accessStr = f"-0x{-access:X}"
             else:
-                accessStr = f"{access:X}"
-            print(f"  {entryAddress:8X} {accessStr:5}(gp) {self.got.localsTable[0]:08X} Lazy resolver")
+                accessStr = f"0x{access:X}"
+            print(f"  {entryAddress:8X} {accessStr:5}($gp) {self.got.localsTable[0]:08X} Lazy resolver")
             entryAddress += 4
 
             print()
 
             print(f" Local entries:")
-            print(f"   Address {'Access':>9}  Initial")
+            print(f"   Address {'Access':>12}  Initial")
             for x in self.got.localsTable[1:]:
                 access = entryAddress - gpValue
                 if access < 0:
-                    accessStr = f"-{-access:X}"
+                    accessStr = f"-0x{-access:X}"
                 else:
-                    accessStr = f"{access:X}"
-                print(f"  {entryAddress:8X} {accessStr:5}(gp) {x:08X}")
+                    accessStr = f"0x{access:X}"
+                print(f"  {entryAddress:8X} {accessStr:5}($gp) {x:08X}")
                 entryAddress += 4
 
             print()
 
             print(f" Global entries:")
-            print(f"  {'Address':>8} {'Access':>9}  Initial Sym.Val. Type    {'Ndx':12} Name")
+            print(f"  {'Address':>8} {'Access':>12}  Initial Sym.Val. Type    {'Ndx':12} Name")
             for gotEntry in self.got.globalsTable:
                 access = entryAddress - gpValue
                 if access < 0:
-                    accessStr = f"-{-access:X}"
+                    accessStr = f"-0x{-access:X}"
                 else:
-                    accessStr = f"{access:X}"
+                    accessStr = f"0x{access:X}"
                 entryType = Elf32SymbolTableType(gotEntry.symEntry.stType)
-                ndx = Elf32SectionHeaderNumber(gotEntry.symEntry.shndx)
+                ndx: str = f"0x{gotEntry.symEntry.shndx:X}"
+                shndx = Elf32SectionHeaderNumber.fromValue(gotEntry.symEntry.shndx)
+                if shndx is not None:
+                    ndx = shndx.name
                 symName = ""
                 if self.dynstr is not None:
                     symName = self.dynstr[gotEntry.symEntry.name]
-                print(f"  {entryAddress:8X} {accessStr:5}(gp) {gotEntry.getAddress():08X} {gotEntry.symEntry.value:08X} {entryType.name:7} {ndx.name:12} {symName}")
+                print(f"  {entryAddress:8X} {accessStr:5}($gp) {gotEntry.getAddress():08X} {gotEntry.symEntry.value:08X} {entryType.name:7} {ndx:12} {symName}")
                 entryAddress += 4
 
             print()
