@@ -17,7 +17,7 @@ from . import SectionBase
 
 class SectionText(SectionBase):
     def __init__(self, context: common.Context, vromStart: int, vromEnd: int, vram: int, filename: str, array_of_bytes: bytearray, segmentVromStart: int, overlayCategory: str|None):
-        super().__init__(context, vromStart, vromEnd, vram, filename, array_of_bytes, common.FileSectionType.Text, segmentVromStart, overlayCategory)
+        super().__init__(context, vromStart, vromEnd, vram, filename, common.Utils.bytesToWords(array_of_bytes, vromStart, vromEnd), common.FileSectionType.Text, segmentVromStart, overlayCategory)
 
         self.instrCat: rabbitizer.Enum = rabbitizer.InstrCategory.CPU
 
@@ -39,17 +39,16 @@ class SectionText(SectionBase):
             instrsList.append(instr)
         return instrsList
 
-    def analyze(self):
+
+    def _findFunctions(self, instrsList: list[rabbitizer.Instruction]):
         functionEnded = False
         farthestBranch = 0
         funcsStartsList = [0]
         unimplementedInstructionsFuncList = []
 
-        instrsList = self.wordListToInstructions(self.words, self.getVramOffset(0), self.instrCat)
-
         instructionOffset = 0
         currentInstructionStart = 0
-        currentFunctionSym = self.getSymbol(self.getVramOffset(instructionOffset), tryPlusOffset=False)
+        currentFunctionSym = self.getSymbol(self.getVramOffset(instructionOffset), tryPlusOffset=False, checkGlobalSegment=False)
 
         isLikelyHandwritten = self.isHandwritten
 
@@ -80,7 +79,7 @@ class SectionText(SectionBase):
                     isboundary = True
 
                 currentInstructionStart = instructionOffset
-                currentFunctionSym = self.getSymbol(self.getVramOffset(instructionOffset), tryPlusOffset=False)
+                currentFunctionSym = self.getSymbol(self.getVramOffset(instructionOffset), tryPlusOffset=False, checkGlobalSegment=False)
 
                 funcsStartsList.append(index)
                 unimplementedInstructionsFuncList.append(not isInstrImplemented)
@@ -95,7 +94,7 @@ class SectionText(SectionBase):
                 isLikelyHandwritten = instr.isLikelyHandwritten()
 
             if instr.isBranch() or instr.isUnconditionalBranch():
-                branchOffset = instr.getGenericBranchOffset(currentVram)
+                branchOffset = instr.getBranchOffsetGeneric()
                 if branchOffset > farthestBranch:
                     # keep track of the farthest branch target
                     farthestBranch = branchOffset
@@ -109,7 +108,7 @@ class SectionText(SectionBase):
                         while j >= 0:
                             if (branchOffset + instructionOffset) < funcsStartsList[j] * 4:
                                 vram = self.getVramOffset(funcsStartsList[j]*4)
-                                funcSymbol = self.getSymbol(vram, tryPlusOffset=False)
+                                funcSymbol = self.getSymbol(vram, tryPlusOffset=False, checkGlobalSegment=False)
                                 if funcSymbol is not None and funcSymbol.isTrustableFunction(self.instrCat == rabbitizer.InstrCategory.RSP):
                                     j -= 1
                                     continue
@@ -119,7 +118,7 @@ class SectionText(SectionBase):
                                 break
                             j -= 1
 
-            elif instr.isJType():
+            elif instr.isJumpWithAddress():
                 target = instr.getInstrIndexAsVram()
                 if self.instrCat != rabbitizer.InstrCategory.RSP:
                     if target >= 0x84000000:
@@ -127,29 +126,43 @@ class SectionText(SectionBase):
                         isLikelyHandwritten = True
                 self.addFunction(target, isAutogenerated=True)
 
-            if not (farthestBranch > 0) and instr.isJump():
-                if instr.isJrRa():
-                    functionEnded = True
-                elif instr.isJrNotRa():
-                    pass
-                elif not instr.doesLink():
-                    if isLikelyHandwritten or self.instrCat == rabbitizer.InstrCategory.RSP:
-                        functionEnded = True
-
-            funcSymbol = self.getSymbol(currentVram + 8, tryPlusOffset=False)
-            if funcSymbol is not None and funcSymbol.isTrustableFunction(self.instrCat == rabbitizer.InstrCategory.RSP):
-                if funcSymbol.vromAddress is None or self.getVromOffset(instructionOffset+8) == funcSymbol.vromAddress:
-                    functionEnded = True
-
+            # Try to find the end of the function
             if currentFunctionSym is not None and currentFunctionSym.size is not None:
+                # If the function has a size set by the user then only use that and ignore the other ways of determining function-ends
                 if instructionOffset + 8 == currentInstructionStart + currentFunctionSym.getSize():
                     functionEnded = True
+            else:
+                if not (farthestBranch > 0) and instr.isJump():
+                    if instr.isJrRa():
+                        # Found a jr $ra and there are no branches outside of this function
+                        functionEnded = True
+                    elif instr.isJrNotRa():
+                        # Usually jumptables, ignore
+                        pass
+                    elif not instr.doesLink():
+                        if isLikelyHandwritten or self.instrCat == rabbitizer.InstrCategory.RSP:
+                            # I don't remember the reasoning of this condition...
+                            functionEnded = True
+
+                # If there's another function after this then the current function has ended
+                funcSymbol = self.getSymbol(currentVram + 8, tryPlusOffset=False, checkGlobalSegment=False)
+                if funcSymbol is not None and funcSymbol.isTrustableFunction(self.instrCat == rabbitizer.InstrCategory.RSP):
+                    if funcSymbol.vromAddress is None or self.getVromOffset(instructionOffset+8) == funcSymbol.vromAddress:
+                        functionEnded = True
 
             index += 1
             farthestBranch -= 4
             instructionOffset += 4
 
         unimplementedInstructionsFuncList.append(not isInstrImplemented)
+        return funcsStartsList, unimplementedInstructionsFuncList
+
+
+    def analyze(self):
+        instrsList = self.wordListToInstructions(self.words, self.getVramOffset(0), self.instrCat)
+        nInstr = len(instrsList)
+
+        funcsStartsList, unimplementedInstructionsFuncList = self._findFunctions(instrsList)
 
         i = 0
         startsCount = len(funcsStartsList)
@@ -163,13 +176,6 @@ class SectionText(SectionBase):
             if start >= end:
                 break
 
-            # TODO: wire up back
-            # funcName = f"func_{i}"
-            # if len(self.context.offsetSymbols[self.sectionType]) > 0:
-            #     possibleFuncName = self.context.getOffsetSymbol(start*4, self.sectionType)
-            #     if possibleFuncName is not None:
-            #         funcName = possibleFuncName.getName()
-
             localOffset = start*4
             vram = self.getVramOffset(localOffset)
 
@@ -177,8 +183,8 @@ class SectionText(SectionBase):
             vromEnd = vrom + (end - start)*4
 
             if common.GlobalConfig.DISASSEMBLE_UNKNOWN_INSTRUCTIONS or not hasUnimplementedIntrs:
-                funcSymbol = self.addFunction(vram, isAutogenerated=True, symbolVrom=vrom)
-            elif common.GlobalConfig.ADD_NEW_SYMBOLS:
+                self.addFunction(vram, isAutogenerated=True, symbolVrom=vrom)
+            else:
                 self.addSymbol(vram, sectionType=self.sectionType, isAutogenerated=True, symbolVrom=vrom)
 
             self.symbolsVRams.add(vram)
