@@ -22,12 +22,12 @@ class SymbolBss(SymbolBase):
         return self.spaceSize // 4
 
     def disassembleAsBss(self) -> str:
-        output = self.getLabel()
+        output = self.getLabelFromSymbol(self.contextSym)
         if common.GlobalConfig.ASM_DATA_SYM_AS_LABEL:
             output += f"{self.getName()}:" + common.GlobalConfig.LINE_ENDS
         output += self.generateAsmLineComment(0)
         output += f" .space 0x{self.spaceSize:02X}" + common.GlobalConfig.LINE_ENDS
         return output
 
-    def disassemble(self) -> str:
+    def disassemble(self, migrate: bool=False, useGlobalLabel: bool=True) -> str:
         return self.disassembleAsBss()
