@@ -55,7 +55,7 @@ class SymbolFunction(SymbolText):
 
         regsTracker = rabbitizer.RegistersTracker(trackedRegistersOriginal)
 
-        self.instrAnalyzer.processInstr(regsTracker, instr, instructionOffset, currentVram, None, self.context.got)
+        self.instrAnalyzer.processInstr(regsTracker, instr, instructionOffset, currentVram, None)
 
         if instructionOffset in self.branchesTaken:
             return
@@ -66,7 +66,7 @@ class SymbolFunction(SymbolText):
             prevTargetInstr = self.instructions[branch//4 - 1]
             targetInstr = self.instructions[branch//4]
 
-            self.instrAnalyzer.processInstr(regsTracker, targetInstr, branch, self.getVramOffset(branch), prevTargetInstr, self.context.got)
+            self.instrAnalyzer.processInstr(regsTracker, targetInstr, branch, self.getVramOffset(branch), prevTargetInstr)
 
             if prevTargetInstr.isUnconditionalBranch():
                 return
@@ -77,48 +77,105 @@ class SymbolFunction(SymbolText):
             branch += 4
 
     def _processElfRelocSymbols(self):
-        if len(self.context.relocSymbols[common.FileSectionType.Text]) == 0:
+        if not self.context.doesSectionHasRelocs(self.sectionType):
             return
 
-        # Process reloc symbols (probably from a .elf file)
+        # Process reloc symbols
         instructionOffset = 0
-        inFileOffset = self.inFileOffset
+        vram = self.vram
         for instr in self.instructions:
-            relocSymbol = self.context.getRelocSymbol(inFileOffset, common.FileSectionType.Text)
-            if relocSymbol is not None:
-                if relocSymbol.name is not None and relocSymbol.name.startswith("."):
-                    sectType = common.FileSectionType.fromStr(relocSymbol.name)
+            relocInfo = self.context.getRelocInfo(vram, self.sectionType)
+            if relocInfo is not None:
+                if relocInfo.referencedSectionVram is not None:
+                    hiOffset = self.instrAnalyzer.lowToHiDict.get(instructionOffset)
+                    if hiOffset is not None:
+                        self.instrAnalyzer.symbolInstrOffset[hiOffset] = self.instrAnalyzer.symbolInstrOffset[instructionOffset]
 
-                    if instructionOffset in self.instrAnalyzer.symbolInstrOffset:
-                        if instructionOffset in self.instrAnalyzer.referencedJumpTableOffsets:
-                            # Jump tables
-                            addressOffset = self.instrAnalyzer.symbolInstrOffset[instructionOffset]
-                            if relocSymbol.name != ".rodata":
-                                common.Utils.eprint(f"Warning. Jumptable referenced in reloc does not have '.rodata' as its name")
-                            contextOffsetSym = self.context.addOffsetJumpTable(addressOffset, sectType)
-                            contextOffsetSym.referenceCounter += 1
-                            contextOffsetSym.referenceFunctions.add(self.contextSym)
-                            if contextOffsetSym.name is not None:
-                                relocSymbol.name = contextOffsetSym.name
-                            self.instrAnalyzer.symbolInstrOffset[instructionOffset] = 0
-                            if instructionOffset in self.instrAnalyzer.lowToHiDict:
-                                luiOffset = self.instrAnalyzer.lowToHiDict[instructionOffset]
-                                otherReloc = self.context.getRelocSymbol(self.inFileOffset+luiOffset, common.FileSectionType.Text)
-                                if otherReloc is not None:
-                                    otherReloc.name = relocSymbol.name
-                                    self.instrAnalyzer.symbolInstrOffset[luiOffset] = 0
-                        else:
-                            addressOffset = self.instrAnalyzer.symbolInstrOffset[instructionOffset]
-                            relocName = f"{relocSymbol.name}_{addressOffset:06X}"
-                            # print(relocName, addressOffset, instr)
-                            contextOffsetSym = common.ContextOffsetSymbol(addressOffset, relocName, sectType)
-                            self.context.offsetSymbols[sectType][addressOffset] = contextOffsetSym
-                            relocSymbol.name = relocName
-                            self.instrAnalyzer.symbolInstrOffset[instructionOffset] = 0
-            inFileOffset += 4
+                    if relocInfo.referencedSection == common.FileSectionType.Rodata:
+                        symbolVram = relocInfo.referencedSectionVram
+                        if instr.hasOperandAlias(rabbitizer.OperandType.cpu_immediate):
+                            if instructionOffset in self.instrAnalyzer.symbolInstrOffset:
+                                addressOffset = self.instrAnalyzer.symbolInstrOffset[instructionOffset]
+                                symbolVram += addressOffset
+                            else:
+                                symbolVram += instr.getProcessedImmediate()
+                        contextSym = self.addSymbol(symbolVram, isAutogenerated=True)
+                        if relocInfo.isStatic:
+                            contextSym._isStatic = relocInfo.isStatic
+
+            vram += 4
             instructionOffset += 4
 
 
+    def _postProcessGotAccesses(self):
+        if not common.GlobalConfig.PIC:
+            return
+
+        globalGotOffsets: set[int] = set()
+
+        for gotAccessOffset, gotAccess in self.instrAnalyzer.gotAccessAddresses.items():
+            gotEntry, inGlobalTable = self.context.got.getGotSymEntry(gotAccess)
+            if gotEntry is None or inGlobalTable is None:
+                continue
+
+            gotAddress = gotEntry.address
+            if inGlobalTable:
+                globalGotOffsets.add(gotAccessOffset)
+                if gotEntry.contextSym is not None:
+                    contextSym = gotEntry.contextSym
+                else:
+                    contextSym = self.addSymbol(gotEntry.address, isAutogenerated=True)
+                contextSym.isGot = True
+                contextSym.isGotGlobal = True
+                loOffset = self.instrAnalyzer.hiToLowDict.get(gotAccessOffset)
+                if loOffset is not None:
+                    if loOffset in self.instrAnalyzer.symbolLoInstrOffset:
+                        del self.instrAnalyzer.symbolLoInstrOffset[loOffset]
+                    if loOffset in self.instrAnalyzer.symbolInstrOffset:
+                        del self.instrAnalyzer.symbolInstrOffset[loOffset]
+            else:
+                # GOT-locals need to be paired
+                loOffset = self.instrAnalyzer.hiToLowDict.get(gotAccessOffset)
+                if loOffset is not None:
+                    loInstr = self.instructions[loOffset//4]
+                    gotAddress += loInstr.getProcessedImmediate()
+                    self.instrAnalyzer.symbolInstrOffset[loOffset] = gotAddress
+
+                    symAccess = self.instrAnalyzer.symbolTypesOffsets.get(loOffset)
+                    if symAccess is not None:
+                        if gotAddress not in self.instrAnalyzer.possibleSymbolTypes:
+                            self.instrAnalyzer.possibleSymbolTypes[gotAddress] = dict()
+                        if symAccess not in self.instrAnalyzer.possibleSymbolTypes[gotAddress]:
+                            self.instrAnalyzer.possibleSymbolTypes[gotAddress][symAccess] = 0
+                        self.instrAnalyzer.possibleSymbolTypes[gotAddress][symAccess] += 1
+
+                contextSym = self.addSymbol(gotAddress, isAutogenerated=True)
+                contextSym.isGot = True
+                contextSym.isGotLocal = True
+            self.instrAnalyzer.symbolInstrOffset[gotAccessOffset] = gotAddress
+            self.instrAnalyzer.referencedVrams.add(gotAddress)
+
+        for loOffset, symVram in self.instrAnalyzer.symbolLoInstrOffset.items():
+            hiOffset = self.instrAnalyzer.lowToHiDict.get(loOffset)
+            if hiOffset is not None and hiOffset in self.instrAnalyzer.gotAccessAddresses:
+                if hiOffset in globalGotOffsets:
+                    if loOffset in self.instrAnalyzer.symbolInstrOffset:
+                        del self.instrAnalyzer.symbolInstrOffset[loOffset]
+                    continue
+
+                if hiOffset in self.instrAnalyzer.symbolInstrOffset:
+                    symVram = self.instrAnalyzer.symbolInstrOffset[hiOffset]
+                    self.instrAnalyzer.symbolLoInstrOffset[loOffset] = symVram
+
+                if loOffset in self.instrAnalyzer.referencedJumpTableOffsets:
+                    self.instrAnalyzer.referencedJumpTableOffsets[loOffset] = symVram
+
+                if loOffset in self.instrAnalyzer.indirectFunctionCallOffsets:
+                    self.instrAnalyzer.indirectFunctionCallOffsets[loOffset] = symVram
+
+        return
+
+
     def analyze(self):
         if not common.GlobalConfig.DISASSEMBLE_UNKNOWN_INSTRUCTIONS and self.hasUnimplementedIntrs:
             offset = 0
@@ -148,7 +205,7 @@ class SymbolFunction(SymbolText):
                 return
 
             if not prevInstr.isBranchLikely() and not prevInstr.isUnconditionalBranch():
-                self.instrAnalyzer.processInstr(regsTracker, instr, instructionOffset, currentVram, prevInstr, self.context.got)
+                self.instrAnalyzer.processInstr(regsTracker, instr, instructionOffset, currentVram, prevInstr)
 
             # look-ahead symbol finder
             self._lookAheadSymbolFinder(instr, prevInstr, instructionOffset, regsTracker)
@@ -159,6 +216,7 @@ class SymbolFunction(SymbolText):
 
         self.instrAnalyzer.printSymbolFinderDebugInfo_UnpairedLuis()
 
+        self._postProcessGotAccesses()
         self._processElfRelocSymbols()
 
         # Branches
@@ -187,7 +245,12 @@ class SymbolFunction(SymbolText):
             if patchedAddress is not None:
                 symVram = patchedAddress
 
-            symAccess = self.instrAnalyzer.possibleSymbolTypes.get(symVram, None)
+            symAccessDict = self.instrAnalyzer.possibleSymbolTypes.get(symVram, dict())
+            symAccess = None
+            if len(symAccessDict) == 1:
+                # Infer type info if there's only one access type
+                symAccess = list(symAccessDict)[0]
+
             contextSym = self.getSymbol(symVram)
             if contextSym is None:
                 contextSym = self.addSymbol(symVram, isAutogenerated=True)
@@ -195,44 +258,31 @@ class SymbolFunction(SymbolText):
                 # TODO: do this in a less ugly way
                 if contextSym.address != symVram:
                     if contextSym.address % 4 != 0 or symVram % 4 != 0:
-                        if contextSym.getType() in {"u16", "s16", "u8", "u8"} or (symAccess is not None and symAccess[0] in {rabbitizer.AccessType.BYTE, rabbitizer.AccessType.SHORT}):
+                        if contextSym.getType() in {"u16", "s16", "u8", "u8"} or (symAccess is not None and symAccess.accessType in {rabbitizer.AccessType.BYTE, rabbitizer.AccessType.SHORT}):
                             if not (contextSym.getSize() > 4):
                                 if contextSym.size is None or symVram >= contextSym.address + contextSym.size:
                                     if symAccess is not None:
-                                        contextSym.setAccessTypeIfUnset(symAccess[0], symAccess[1])
+                                        contextSym.setAccessTypeIfUnset(symAccess.accessType, symAccess.unsignedMemoryAccess)
+                                    contextSym.setFirstLoAccessIfUnset(loOffset)
                                     contextSym = self.addSymbol(symVram, isAutogenerated=True)
 
             contextSym.referenceCounter += 1
             contextSym.referenceFunctions.add(self.contextSym)
             if symAccess is not None:
-                contextSym.setAccessTypeIfUnset(symAccess[0], symAccess[1])
+                contextSym.setAccessTypeIfUnset(symAccess.accessType, symAccess.unsignedMemoryAccess)
+            contextSym.setFirstLoAccessIfUnset(loOffset)
 
         # Jump tables
-        for targetVram in self.instrAnalyzer.jumpRegisterIntrOffset.values():
+        for targetVram in self.instrAnalyzer.referencedJumpTableOffsets.values():
             self.addJumpTable(targetVram, isAutogenerated=True)
 
-        for gpLoadOffset in self.instrAnalyzer.gpLoads.keys():
-            gpSymbolAddress = self.instrAnalyzer.symbolHiInstrOffset.get(gpLoadOffset, None)
-            if gpSymbolAddress is not None:
-                # In case this is a paired %got symbol then tell the upper gp access the real address
-                self.instrAnalyzer.symbolInstrOffset[gpLoadOffset] = gpSymbolAddress
-            else:
-                # This $gp load was not paired with any other symbol
-                gpSymbolAddress = self.instrAnalyzer.symbolInstrOffset.get(gpLoadOffset)
-            if gpSymbolAddress is not None:
-                contextSym = self.getSymbol(gpSymbolAddress, tryPlusOffset=False)
-                if contextSym is not None:
-                    contextSym.isGot = True
-            elif common.GlobalConfig.GP_VALUE is not None:
-                gpLoadInstr = self.instrAnalyzer.gpLoads[gpLoadOffset]
-                gpSymbolAddress = common.GlobalConfig.GP_VALUE + gpLoadInstr.getProcessedImmediate()
-                self.instrAnalyzer.symbolInstrOffset[gpLoadOffset] = gpSymbolAddress
-                self.addSymbol(gpSymbolAddress, isAutogenerated=True)
 
         if self.isLikelyHandwritten:
             for instr in self.instructions:
                 instr.inHandwrittenFunction = self.isLikelyHandwritten
 
+        self.contextSym.autodetectedSize = self.sizew*4
+
 
     def countExtraPadding(self) -> int:
         count = 0
@@ -326,34 +376,50 @@ class SymbolFunction(SymbolText):
             del self.instructions[first_nop:]
         return was_updated
 
+    _percentRel = {
+        elf32.Elf32Relocs.MIPS_HI16.value:      f"%hi",
+        elf32.Elf32Relocs.MIPS_LO16.value:      f"%lo",
+        elf32.Elf32Relocs.MIPS_GPREL16.value:   f"%gp_rel",
+        elf32.Elf32Relocs.MIPS_GOT16.value:     f"%got",
+        elf32.Elf32Relocs.MIPS_CALL16.value:    f"%call16",
+        elf32.Elf32Relocs.MIPS_GOT_HI16.value:  f"%got_hi",
+        elf32.Elf32Relocs.MIPS_GOT_LO16.value:  f"%got_lo",
+        elf32.Elf32Relocs.MIPS_CALL_HI16.value: f"%call_hi",
+        elf32.Elf32Relocs.MIPS_CALL_LO16.value: f"%call_lo",
+    }
 
-    def generateHiLoStr(self, instr: rabbitizer.Instruction, symName: str, symbol: common.ContextSymbol|None, relocInfo: common.ContextRelocInfo|None=None) -> str:
+    def generateHiLoStr(self, instr: rabbitizer.Instruction, instrOffset: int, symName: str, symbol: common.ContextSymbol|None, relocInfo: common.ContextRelocInfo|None=None, gotHiLo: bool=False) -> str:
         if relocInfo is not None:
-            if relocInfo.relocType == elf32.Elf32Relocs.MIPS_HI16.value:
-                return f"%hi({symName})"
-            if relocInfo.relocType == elf32.Elf32Relocs.MIPS_LO16.value:
-                return f"%lo({symName})"
-            if relocInfo.relocType == elf32.Elf32Relocs.MIPS_GPREL16.value:
-                return f"%gp_rel({symName})"
-            if relocInfo.relocType == elf32.Elf32Relocs.MIPS_GOT16.value:
-                return f"%got({symName})"
-            if relocInfo.relocType == elf32.Elf32Relocs.MIPS_CALL16.value:
-                return f"%call16({symName})"
+            percentStr = self._percentRel.get(relocInfo.relocType, None)
+            if percentStr is not None:
+                return f"{percentStr}({symName})"
             common.Utils.eprint(f"generateHiLoStr: Warning: Unhandled relocType '{relocInfo.relocType}'")
 
         if instr.canBeHi():
+            if common.GlobalConfig.PIC:
+                if symbol is not None and gotHiLo:
+                    if symbol.isGotGlobal and symbol.type == common.SymbolSpecialType.function:
+                        return f"%call_hi({symName})"
+                    else:
+                        return f"%got_hi({symName})"
             return f"%hi({symName})"
 
         if instr.rs in {rabbitizer.RegGprO32.gp, rabbitizer.RegGprN32.gp}:
             if common.GlobalConfig.PIC:
                 if symbol is not None:
-                    if symbol.isGotGlobal and symbol.type == common.SymbolSpecialType.function:
+                    if symbol.isGotGlobal and symbol.type == common.SymbolSpecialType.function and instrOffset in self.instrAnalyzer.indirectFunctionCallOffsets:
                         return f"%call16({symName})"
                     elif symbol.isGot:
                         return f"%got({symName})"
             if not common.GlobalConfig.PIC:
                 return f"%gp_rel({symName})"
 
+        if common.GlobalConfig.PIC:
+            if symbol is not None and gotHiLo:
+                if symbol.isGotGlobal and symbol.type == common.SymbolSpecialType.function:
+                    return f"%call_lo({symName})"
+                else:
+                    return f"%got_lo({symName})"
         return f"%lo({symName})"
 
     def generateHiLoConstantStr(self, constantValue: int, currentInstr: rabbitizer.Instruction, loInstr: rabbitizer.Instruction|None) -> str|None:
@@ -386,18 +452,33 @@ class SymbolFunction(SymbolText):
 
 
     def getImmOverrideForInstruction(self, instr: rabbitizer.Instruction, instructionOffset: int) -> str|None:
-        if len(self.context.relocSymbols[self.sectionType]) > 0:
-            # Check possible symbols using reloc information (probably from a .o elf file)
-            possibleImmOverride = self.context.getRelocSymbol(self.inFileOffset + instructionOffset, self.sectionType)
-            if possibleImmOverride is not None:
-                auxOverride = possibleImmOverride.getName()
+        # Check possible symbols using reloc information
+        relocInfo = self.context.getRelocInfo(self.vram + instructionOffset, self.sectionType)
+        if relocInfo is not None:
+            if relocInfo.referencedSectionVram is not None:
+                relocVram = relocInfo.referencedSectionVram
+                addend = 0
                 if instr.hasOperandAlias(rabbitizer.OperandType.cpu_immediate):
                     if instructionOffset in self.instrAnalyzer.symbolInstrOffset:
-                        addressOffset = self.instrAnalyzer.symbolInstrOffset[instructionOffset]
-                        auxOverride = possibleImmOverride.getNamePlusOffset(addressOffset)
+                        addend = self.instrAnalyzer.symbolInstrOffset[instructionOffset]
+                    else:
+                        addend = instr.getProcessedImmediate()
 
-                    auxOverride = self.generateHiLoStr(instr, auxOverride, None, relocInfo=possibleImmOverride)
-                return auxOverride
+                contextSym = self.getSymbol(relocVram+addend, checkUpperLimit=False)
+                if contextSym is not None:
+                    symName = contextSym.getSymbolPlusOffset(relocVram+addend)
+                    return self.generateHiLoStr(instr, instructionOffset, symName, contextSym, relocInfo=relocInfo)
+
+            auxOverride = relocInfo.getNamePlusOffset(0)
+            if instr.hasOperandAlias(rabbitizer.OperandType.cpu_immediate):
+                if instructionOffset in self.instrAnalyzer.symbolInstrOffset:
+                    addressOffset = self.instrAnalyzer.symbolInstrOffset[instructionOffset]
+                    auxOverride = relocInfo.getNamePlusOffset(addressOffset)
+                else:
+                    auxOverride = relocInfo.getNamePlusOffset(instr.getProcessedImmediate())
+
+                auxOverride = self.generateHiLoStr(instr, instructionOffset, auxOverride, None, relocInfo=relocInfo)
+            return auxOverride
 
         if instr.isBranch() or instr.isUnconditionalBranch():
             if not common.GlobalConfig.IGNORE_BRANCHES:
@@ -410,7 +491,7 @@ class SymbolFunction(SymbolText):
         elif instr.hasOperandAlias(rabbitizer.OperandType.cpu_immediate):
             # .cpload directive is meant to use the `_gp_disp` pseudo-symbol
             if instructionOffset in self.instrAnalyzer.cploadOffsets:
-                return self.generateHiLoStr(instr, "_gp_disp", None)
+                return self.generateHiLoStr(instr, instructionOffset, "_gp_disp", None)
 
             if not self.pointersRemoved and instructionOffset in self.instrAnalyzer.symbolInstrOffset:
                 address = self.instrAnalyzer.symbolInstrOffset[instructionOffset]
@@ -418,6 +499,7 @@ class SymbolFunction(SymbolText):
                 if address in self.context.bannedSymbols:
                     return None
 
+                symbol: common.ContextSymbol|None = None
                 loInstr = instr
                 instrVram = self.getVramOffset(instructionOffset)
                 if instr.canBeHi():
@@ -432,22 +514,34 @@ class SymbolFunction(SymbolText):
                 if patchedAddress is not None:
                     symbol = self.getSymbol(patchedAddress, tryPlusOffset=True, checkUpperLimit=False)
                 else:
-                    symbol = self.getSymbol(address, tryPlusOffset=True)
+                    gotAccess = self.instrAnalyzer.gotAccessAddresses.get(instructionOffset)
+                    if gotAccess is not None:
+                        gotEntry, inGlobalTable = self.context.got.getGotSymEntry(gotAccess)
+                        if gotEntry is not None:
+                            symbol = gotEntry.contextSym
+                    if symbol is None:
+                        symbol = self.getSymbol(address, tryPlusOffset=True)
+
+                gotHiLo = False
+                if symbol is None and address < 0 and common.GlobalConfig.PIC and common.GlobalConfig.GP_VALUE is not None:
+                    # Negative pointer may mean it is a weird GOT access
+                    gotAccess = common.GlobalConfig.GP_VALUE + address
+                    gotEntry, inGlobalTable = self.context.got.getGotSymEntry(gotAccess)
+                    if gotEntry is not None:
+                        if gotEntry.contextSym is not None:
+                            symbol = gotEntry.contextSym
+                        else:
+                            symbol = self.getSymbol(gotEntry.address)
+                        gotHiLo = True
+                        address = gotEntry.address
 
                 if symbol is not None:
-                    symName = symbol.getSymbolPlusOffset(address)
-                    if common.GlobalConfig.PIC:
-                        if instructionOffset not in self.instrAnalyzer.gpLoads:
-                            if address in self.context.got.globalsTable:
-                                # addend = instr.getProcessedImmediate()
-                                # if addend < 0:
-                                #     symName = f"{symName} - 0x{-addend:X}"
-                                # elif addend > 0:
-                                #     symName = f"{symName} + 0x{addend:X}"
-                                # # do nothing if addend is zero
-                                return None
+                    if symbol.isGotGlobal:
+                        if instructionOffset not in self.instrAnalyzer.gotAccessAddresses and not gotHiLo:
+                            return None
 
-                    return self.generateHiLoStr(instr, symName, symbol)
+                    symName = symbol.getSymbolPlusOffset(address)
+                    return self.generateHiLoStr(instr, instructionOffset, symName, symbol, gotHiLo=gotHiLo)
                 return self.generateHiLoConstantStr(address, instr, loInstr)
 
             elif instructionOffset in self.instrAnalyzer.constantInstrOffset:
@@ -455,7 +549,7 @@ class SymbolFunction(SymbolText):
 
                 symbol = self.getConstant(constant)
                 if symbol is not None:
-                    return self.generateHiLoStr(instr, symbol.getName(), symbol)
+                    return self.generateHiLoStr(instr, instructionOffset, symbol.getName(), symbol)
 
                 # Pretend this pair is a constant
                 loInstr = instr
@@ -467,7 +561,7 @@ class SymbolFunction(SymbolText):
                     return generatedStr
 
                 if common.GlobalConfig.SYMBOL_FINDER_FILTERED_ADDRESSES_AS_HILO:
-                    return self.generateHiLoStr(instr, f"0x{constant:X}", None)
+                    return self.generateHiLoStr(instr, instructionOffset, f"0x{constant:X}", None)
 
             if instr.canBeHi():
                 # Unpaired LUI
@@ -480,24 +574,20 @@ class SymbolFunction(SymbolText):
 
         return None
 
-    def getLabelForOffset(self, instructionOffset: int) -> str:
+    def getLabelForOffset(self, instructionOffset: int, migrate: bool=False) -> str:
         if common.GlobalConfig.IGNORE_BRANCHES or instructionOffset == 0:
             # Skip over this function to avoid duplication
             return ""
 
         currentVram = self.getVramOffset(instructionOffset)
         labelSym = self.getSymbol(currentVram, tryPlusOffset=False)
-        if labelSym is None and len(self.context.offsetJumpTablesLabels) > 0:
-            labelSym = self.context.getOffsetGenericLabel(self.inFileOffset+instructionOffset, common.FileSectionType.Text)
-        if labelSym is None and len(self.context.offsetSymbols[self.sectionType]) > 0:
-            labelSym = self.context.getOffsetSymbol(self.inFileOffset+instructionOffset, common.FileSectionType.Text)
 
-        if labelSym is None or labelSym.overlayCategory != self.overlayCategory or labelSym.isGotLocal:
+        if labelSym is None or labelSym.overlayCategory != self.overlayCategory:
             return ""
 
         labelSym.isDefined = True
         labelSym.sectionType = self.sectionType
-        if labelSym.type == common.SymbolSpecialType.function or labelSym.type == common.SymbolSpecialType.jumptablelabel:
+        if labelSym.type == common.SymbolSpecialType.function or (labelSym.type == common.SymbolSpecialType.jumptablelabel and not migrate):
             label = labelSym.getSymbolLabel()
             if label:
                 label += common.GlobalConfig.LINE_ENDS
@@ -541,19 +631,19 @@ class SymbolFunction(SymbolText):
             # don't emit the other instructions which are part of .cpload if the directive was emitted
         return output
 
-    def disassemble(self) -> str:
+    def disassemble(self, migrate: bool=False, useGlobalLabel: bool=True) -> str:
         output = ""
 
         if not common.GlobalConfig.DISASSEMBLE_UNKNOWN_INSTRUCTIONS:
             if self.hasUnimplementedIntrs:
-                return self.disassembleAsData()
+                return self.disassembleAsData(useGlobalLabel=useGlobalLabel)
 
         if self.isLikelyHandwritten:
             if not self.isRsp:
                 # RSP functions are always handwritten, so this is redundant
                 output += "# Handwritten function" + common.GlobalConfig.LINE_ENDS
 
-        output += self.getLabel()
+        output += self.getLabelFromSymbol(self.contextSym)
 
         if common.GlobalConfig.ASM_TEXT_ENT_LABEL:
             output += f"{common.GlobalConfig.ASM_TEXT_ENT_LABEL} {self.getName()}" + common.GlobalConfig.LINE_ENDS
@@ -564,7 +654,7 @@ class SymbolFunction(SymbolText):
         wasLastInstABranch = False
         instructionOffset = 0
         for instr in self.instructions:
-            label = self.getLabelForOffset(instructionOffset)
+            label = self.getLabelForOffset(instructionOffset, migrate=migrate)
             output += label
 
             isCpload = instructionOffset in self.instrAnalyzer.cploadOffsets
@@ -581,6 +671,6 @@ class SymbolFunction(SymbolText):
 
         return output
 
-    def disassembleAsData(self) -> str:
+    def disassembleAsData(self, useGlobalLabel: bool=True) -> str:
         self.words = [instr.getRaw() for instr in self.instructions]
-        return super().disassembleAsData()
+        return super().disassembleAsData(useGlobalLabel=useGlobalLabel)
