@@ -11,6 +11,22 @@ import rabbitizer
 from .... import common
 
 
+@dataclasses.dataclass
+class SymbolTypeInfo:
+    accessType: rabbitizer.Enum
+    """<rabbitizer.AccessType>"""
+    unsignedMemoryAccess: bool
+
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, SymbolTypeInfo):
+            return False
+        return self.accessType == other.accessType and self.unsignedMemoryAccess == other.unsignedMemoryAccess
+
+    # https://stackoverflow.com/a/56915493/6292472
+    def __hash__(self):
+        return hash((self.accessType, self.unsignedMemoryAccess))
+
+
 @dataclasses.dataclass
 class CploadInfo:
     hiOffset: int
@@ -18,6 +34,7 @@ class CploadInfo:
     adduOffset: int|None = None
     reg: rabbitizer.Enum|None = None
 
+
 class InstrAnalyzer:
     def __init__(self, funcVram: int) -> None:
         self.funcVram = funcVram
@@ -48,6 +65,10 @@ class InstrAnalyzer:
         self.jumpRegisterIntrOffset: dict[int, int] = dict()
         self.referencedJumpTableOffsets: dict[int, int] = dict()
 
+        # Jump register (jumptables)
+        self.indirectFunctionCallIntrOffset: dict[int, int] = dict()
+        self.indirectFunctionCallOffsets: dict[int, int] = dict()
+
         # Constants
         self.constantHiInstrOffset: dict[int, int] = dict()
         "key: offset of instruction which is setting the %hi constant, value: constant"
@@ -66,8 +87,9 @@ class InstrAnalyzer:
 
         self.symbolInstrOffset: dict[int, int] = dict()
 
-        self.possibleSymbolTypes: dict[int, tuple[rabbitizer.Enum, bool]] = dict()
-        "key: address, value: (<rabbitizer.AccessType>, unsignedMemoryAccess)"
+        self.possibleSymbolTypes: dict[int, dict[SymbolTypeInfo, int]] = dict()
+        "key: address, value: {<SymbolTypeInfo>: number of times this type appears in code}"
+        self.symbolTypesOffsets: dict[int, SymbolTypeInfo] = dict()
 
         # %hi/%lo pairing
         self.hiToLowDict: dict[int, int] = dict()
@@ -79,7 +101,7 @@ class InstrAnalyzer:
 
         self.nonLoInstrOffsets: set[int] = set()
 
-        self.gpLoads: dict[int, rabbitizer.Instruction] = dict()
+        self.gotAccessAddresses: dict[int, int] = dict()
 
         self.unpairedCploads: list[CploadInfo] = list()
         "cploads which are not yet fully paired"
@@ -98,7 +120,8 @@ class InstrAnalyzer:
         branch = instrOffset + branchOffset
         targetBranchVram = self.funcVram + branch
 
-        self.referencedVrams.add(targetBranchVram)
+        if not common.GlobalConfig.PIC:
+            self.referencedVrams.add(targetBranchVram)
 
         self.branchLabelOffsets.add(branch)
         self.branchInstrOffsets[instrOffset] = targetBranchVram
@@ -113,7 +136,8 @@ class InstrAnalyzer:
         if target >= 0x84000000 or target < 0x80000000:
             self.funcCallOutsideRangesOffsets[instrOffset] = target
 
-        self.referencedVrams.add(target)
+        if not common.GlobalConfig.PIC:
+            self.referencedVrams.add(target)
         self.referencedVramsInstrOffset[instrOffset] = target
 
         self.funcCallInstrOffsets[instrOffset] = target
@@ -139,7 +163,7 @@ class InstrAnalyzer:
         return constant
 
 
-    def pairHiLo(self, hiValue: int|None, luiOffset: int|None, lowerInstr: rabbitizer.Instruction, lowerOffset: int, got: common.GlobalOffsetTable, otherIsGpGot: bool) -> int|None:
+    def pairHiLo(self, hiValue: int|None, luiOffset: int|None, lowerInstr: rabbitizer.Instruction, lowerOffset: int) -> int|None:
         # lui being None means this symbol is a $gp access
         assert (hiValue is None and luiOffset is None) or (hiValue is not None and luiOffset is not None)
 
@@ -161,7 +185,7 @@ class InstrAnalyzer:
                 # IDO does not pair multiples %hi to the same %lo
                 return self.symbolLoInstrOffset[lowerOffset]
 
-            elif common.GlobalConfig.COMPILER in {common.Compiler.GCC, common.Compiler.SN64}:
+            elif common.GlobalConfig.COMPILER in {common.Compiler.GCC, common.Compiler.SN64, common.Compiler.PSYQ}:
                 if luiOffset is None or hiValue is None:
                     return None
 
@@ -196,22 +220,18 @@ class InstrAnalyzer:
 
         if hiValue is not None:
             upperHalf = hiValue
-            if otherIsGpGot and upperHalf in got.globalsTable:
-                lowerHalf = 0
         else:
             assert common.GlobalConfig.GP_VALUE is not None
             upperHalf = common.GlobalConfig.GP_VALUE
 
-            if got.tableStart is not None:
-                return got.getAddress(upperHalf + lowerHalf)
-
         return upperHalf + lowerHalf
 
 
     def processSymbol(self, address: int, luiOffset: int|None, lowerInstr: rabbitizer.Instruction, lowerOffset: int) -> int|None:
         if address <= 0:
             # PS2 seems to do LUI/ADDIU pairs for 0 and -1 for some reason, filter them out
-            return None
+            if not common.GlobalConfig.PIC:
+                return None
 
         # filter out stuff that may not be a real symbol
         filterOut = common.GlobalConfig.SYMBOL_FINDER_FILTER_LOW_ADDRESSES and address < 0x80000000
@@ -232,7 +252,8 @@ class InstrAnalyzer:
                     self.lowToHiDict[lowerOffset] = luiOffset
             return None
 
-        self.referencedVrams.add(address)
+        if not common.GlobalConfig.PIC:
+            self.referencedVrams.add(address)
 
         if lowerOffset not in self.symbolLoInstrOffset:
             self.symbolLoInstrOffset[lowerOffset] = address
@@ -251,32 +272,41 @@ class InstrAnalyzer:
             self.symbolInstrOffset[lowerOffset] = address
             self.referencedVramsInstrOffset[lowerOffset] = address
 
-        self.processSymbolType(address, lowerInstr)
+        self.processSymbolType(address, lowerInstr, lowerOffset)
 
         return address
 
-    def processSymbolType(self, address: int, instr: rabbitizer.Instruction) -> None:
+    def processGotSymbol(self, address: int, instr: rabbitizer.Instruction, instrOffset: int) -> None:
+        if address <= 0:
+            return
+
+        self.gotAccessAddresses[instrOffset] = address
+        return
+
+    def processSymbolType(self, address: int, instr: rabbitizer.Instruction, instrOffset: int) -> None:
         accessType = instr.getAccessType()
         unsignedMemoryAccess = instr.doesUnsignedMemoryAccess()
         if accessType == rabbitizer.AccessType.INVALID:
             return
 
-        if accessType == rabbitizer.AccessType.WORD:
-            if not unsignedMemoryAccess:
-                return
-
         if address not in self.possibleSymbolTypes:
-            self.possibleSymbolTypes[address] = (accessType, unsignedMemoryAccess)
+            self.possibleSymbolTypes[address] = dict()
+        symAccess = SymbolTypeInfo(accessType, unsignedMemoryAccess)
+        if symAccess not in self.possibleSymbolTypes[address]:
+            self.possibleSymbolTypes[address][symAccess] = 0
+        self.possibleSymbolTypes[address][symAccess] += 1
+
+        self.symbolTypesOffsets[instrOffset] = symAccess
 
     def processSymbolDereferenceType(self, regsTracker: rabbitizer.RegistersTracker, instr: rabbitizer.Instruction, instrOffset: int) -> None:
         address = regsTracker.getAddressIfCanSetType(instr, instrOffset)
         if address is None:
             return
 
-        self.processSymbolType(address, instr)
+        self.processSymbolType(address, instr, instrOffset)
 
 
-    def symbolFinder(self, regsTracker: rabbitizer.RegistersTracker, instr: rabbitizer.Instruction, prevInstr: rabbitizer.Instruction|None, instrOffset: int, got: common.GlobalOffsetTable) -> None:
+    def symbolFinder(self, regsTracker: rabbitizer.RegistersTracker, instr: rabbitizer.Instruction, prevInstr: rabbitizer.Instruction|None, instrOffset: int) -> None:
         if instr.canBeHi():
             if prevInstr is None:
                 regsTracker.processLui(instr, instrOffset)
@@ -285,10 +315,8 @@ class InstrAnalyzer:
             self.luiInstrs[instrOffset] = instr
             return
 
-        instrDoesGpLoad = False
         if instr.doesLoad() and instr.rs in {rabbitizer.RegGprO32.gp, rabbitizer.RegGprN32.gp}:
             regsTracker.processGpLoad(instr, instrOffset)
-            instrDoesGpLoad = True
 
         if not instr.canBeLo():
             return
@@ -332,12 +360,15 @@ class InstrAnalyzer:
                         # early return to avoid counting this pairing as a symbol
                         return
 
-        address = self.pairHiLo(upperHalf, luiOffset, instr, instrOffset, got, pairingInfo.isGpGot)
+        address = self.pairHiLo(upperHalf, luiOffset, instr, instrOffset)
         if address is None:
             return
 
-        if instrDoesGpLoad:
-            self.gpLoads[instrOffset] = instr
+        if upperHalf is None:
+            if common.GlobalConfig.PIC:
+                self.processGotSymbol(address, instr, instrOffset)
+                regsTracker.processLo(instr, address, instrOffset)
+                return
 
         address = self.processSymbol(address, luiOffset, instr, instrOffset)
         if address is not None:
@@ -351,10 +382,21 @@ class InstrAnalyzer:
 
             self.referencedJumpTableOffsets[offset] = address
             self.jumpRegisterIntrOffset[instrOffset] = address
-            self.referencedVrams.add(address)
+            if not common.GlobalConfig.PIC:
+                self.referencedVrams.add(address)
+
+    def processJumpAndLinkRegister(self, regsTracker: rabbitizer.RegistersTracker, instr: rabbitizer.Instruction, instrOffset: int) -> None:
+        jrInfo = regsTracker.getJrInfo(instr)
+        if jrInfo is not None:
+            offset, address = jrInfo
+
+            self.indirectFunctionCallOffsets[offset] = address
+            self.indirectFunctionCallIntrOffset[instrOffset] = address
+            if not common.GlobalConfig.PIC:
+                self.referencedVrams.add(address)
 
 
-    def processInstr(self, regsTracker: rabbitizer.RegistersTracker, instr: rabbitizer.Instruction, instrOffset: int, currentVram: int, prevInstr: rabbitizer.Instruction|None, got: common.GlobalOffsetTable) -> None:
+    def processInstr(self, regsTracker: rabbitizer.RegistersTracker, instr: rabbitizer.Instruction, instrOffset: int, currentVram: int, prevInstr: rabbitizer.Instruction|None) -> None:
         if instr.isBranch() or instr.isUnconditionalBranch():
             self.processBranch(instr, instrOffset, currentVram)
 
@@ -362,12 +404,16 @@ class InstrAnalyzer:
             self.processFuncCall(instr, instrOffset)
 
         elif instr.hasOperandAlias(rabbitizer.OperandType.cpu_immediate):
-            self.symbolFinder(regsTracker, instr, prevInstr, instrOffset, got)
+            self.symbolFinder(regsTracker, instr, prevInstr, instrOffset)
             self.processSymbolDereferenceType(regsTracker, instr, instrOffset)
 
         elif instr.isJrNotRa():
             self.processJumpRegister(regsTracker, instr, instrOffset)
 
+        elif instr.isJump() and instr.doesLink():
+            # jalr, implicit not isJumpWithAddress
+            self.processJumpAndLinkRegister(regsTracker, instr, instrOffset)
+
         elif instr.uniqueId == rabbitizer.InstrId.cpu_addu:
             # special check for .cpload
             if len(self.unpairedCploads) > 0:
